
 Let's create a simple Python web server using Flask, connect it to a MySQL database, and set up some APIs to interact with the database. We'll use Docker to containerize both the web server and the database.

Here we are going to create a complete end-to-end package for a Dockerized Python web server with Flask, connected to a MySQL database, and with APIs to store and retrieve JSON data.

### Project Structure

Create a directory in your Azure VM for your project and structure it as follows:

#### **Step 1: Create the main directory**  
```bash
mkdir my-flask-api
cd my-flask-api
```

#### **Step 2: Create the necessary files**  
```bash
touch Dockerfile docker-compose.yml requirements.txt app.py
```

#### **Step 3: Verify the structure**  
```bash
tree my-flask-api
```
*(If `tree` is not installed, you can install it using `sudo apt install tree`.)*  

This will create the following structure:  
```
my-flask-api/
├── Dockerfile
├── docker-compose.yml
├── requirements.txt
└── app.py
```

Now you need to add content in these files! For this you need to use Nano text editor of the system.

#### How to use Nano editor:
```
- Open a file: nano filename

- You can use Arrow Keys to navigate.

- You can copy your content and paste them in the file using right click of your mouse.

Save & Exit: Press Ctrl + X, then Y, then Enter.
```

### Dockerfile

Edit `Dockerfile` with the following content using command `nano Dockerfile`:
```Dockerfile
# Use the official Python image from the Docker Hub
FROM python:3.9-slim

# Set the working directory in the container
WORKDIR /app

# Copy the current directory contents into the container at /app
COPY . /app

# Install any needed packages specified in requirements.txt
RUN pip install --no-cache-dir -r requirements.txt

# Make port 5000 available to the world outside this container
EXPOSE 5000

# Run app.py when the container launches
CMD ["python", "app.py"]
```

### docker-compose.yml

Edit `docker-compose.yml` file using command `nano docker-compose.yml` with the following content:
```yaml
version: '3'
services:
  db:
    image: mysql
    environment:
      MYSQL_ROOT_PASSWORD: my-secret-pw
      MYSQL_DATABASE: testdb
    ports:
      - "3306:3306"
  web:
    build: .
    ports:
      - "5000:5000"
    depends_on:
      - db
```

### requirements.txt

Edit the `requirements.txt` file using command `nano requirements.txt` with the following content:
```text
Flask
mysql-connector-python
```

### app.py

Edit `app.py` file using command `nano app.py` with the following content:
```python
from flask import Flask, request, jsonify
import mysql.connector
import json
import time

app = Flask(__name__)

DB_CONFIG = {
    "host": "db",  # Use the service name from docker-compose.yml
    "user": "root",
    "password": "my-secret-pw",
    "database": "testdb"
}

# Retry MySQL Connection
while True:
    try:
        print("Attempting to connect to MySQL...")
        db = mysql.connector.connect(**DB_CONFIG)
        cursor = db.cursor()
        print("Connected to MySQL successfully!")

        # Ensure the 'users' table exists
        cursor.execute("""
            CREATE TABLE IF NOT EXISTS users (
                id INT AUTO_INCREMENT PRIMARY KEY,
                name VARCHAR(255),
                data JSON
            )
        """)
        db.commit()
        break  # Exit loop on success

    except mysql.connector.Error as err:
        print(f"MySQL connection error: {err}")
        print("Retrying in 5 seconds...")
        time.sleep(5)

@app.route('/users', methods=['POST'])
def add_user():
    data = request.get_json()
    name = data.get('name')
    user_data = data.get('data')

    if not name or user_data is None:
        return jsonify({"error": "Invalid request"}), 400

    try:
        db = mysql.connector.connect(**DB_CONFIG)
        cursor = db.cursor()
        cursor.execute("INSERT INTO users (name, data) VALUES (%s, %s)", (name, json.dumps(user_data)))
        db.commit()
        cursor.close()
        db.close()
        return jsonify({"message": "User added!"}), 201

    except mysql.connector.Error as err:
        return jsonify({"error": str(err)}), 500

@app.route('/users', methods=['GET'])
def get_users():
    try:
        db = mysql.connector.connect(**DB_CONFIG)
        cursor = db.cursor(dictionary=True)  # Fetch data as dictionary
        cursor.execute("SELECT * FROM users")
        users = cursor.fetchall()
        cursor.close()
        db.close()
        return jsonify(users)

    except mysql.connector.Error as err:
        return jsonify({"error": str(err)}), 500

if __name__ == '__main__':
    app.run(host='0.0.0.0', port=5000)

```

### Running the Application

1. **Navigate to the project directory**:
   ```bash
   cd my-flask-api
   ```

2. **Build and run the containers**:
   ```bash
   sudo apt  install docker-compose
   docker-compose up --build
   ```
### Testing the new images created
Use the below command to see the build outcome of above command resulted in creating a new applicaiton image for your or not.
```bash
docker images
```
### Testing the API

1. **Store Data**: Open the another ssh terminal for your VM using MobaXterm on Windows or MAc terminal
                   and execute the below commands.
   ```bash
   curl -X POST http://localhost:5000/users -H "Content-Type: application/json" -d '{"name": "John Doe", "data": {"age": 30, "city": "New York"}}'
   ```

2. **Retrieve Data**:
   ```bash
   curl http://localhost:5000/users
   ```

### Clean Up: In case you reset the whole environment you need to shut down your launched docker containers. 

1. **Stop and remove the containers**:
   ```bash
   docker-compose down
   ```

This setup should give you a fully functional Dockerized Flask application connected to a MySQL database, with APIs to store and retrieve JSON data. 